! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
!+
! Module: emask_mod
!
! Authors:  Hermann Brunner - AIP
!          
! Description: Detection mask creation (from exposure images), marking
! 
! Comments: 
! 
! $Id: emask_mod.f90,v 1.2 2011/02/15 09:27:00 rsaxton Exp $
!- 
module emask_mod

  use dal
  use param  
  use edetect_io_mask  

  implicit none 
  
  private 

  public emask

contains 
   
  !---------------------------------------------------------------------
  subroutine emask
  !---------------------------------------------------------------------
    
    integer :: ix, iy, nreg, ireg
    real(kind=single) :: fracth, gradth, emax, pa
    real(kind=single) :: xr, yr
    character(len=1000) :: ostring
    character(len=1024), dimension(16) :: image
    character(len=16), dimension(16) :: instrume 
    character(len=1024) :: regionfile
    integer, dimension(16) :: band     
    integer, dimension(2) :: npix 
    real(kind=single), dimension(:,:), pointer :: expo
    integer(kind=int32), dimension(:,:), pointer :: x, x2
    real, dimension(max_regions) :: xreg,yreg,rreg,w1reg,w2reg,rotreg
    integer, dimension(max_regions) :: regtype
    real, dimension(max_regions) :: rreg2, sinp, cosp
    integer, dimension(:), pointer :: shape
    logical :: regflag
    
    type(DataSetT) :: set, setO
    type(BlockT) :: maskBlock
!    type(ArrayT) :: earr, marr, marr2
    type(ArrayT) :: earr, marr
     
! threshold parameter [0.0<parameter<1.0]
         
    fracth = realParameter("threshold1")
    gradth = realParameter("threshold2")

! names of exposure images (A,B,C) one per instrument
! for the moment, read just the first one

!   call get_files ("expo_image", image, instrume, band, .true.)    

    image(1) = stringParameter("expimageset")
    set = dataSet(image(1),READ)
    earr = array(set,"PRIMARY",READ)
    expo => real32Array2Data(earr) 

    shape => dimensions(earr)
    npix(1) = shape(0)
    npix(2) = shape(1)
      
! bbname of detection mask file
      
    regflag= booleanParameter("withregionset")
    if (regflag) then
      regionfile=stringParameter("regionset")
      call get_regions(regionfile,nreg,xreg,yreg,rreg,w1reg,w2reg,rotreg,regtype)
    endif

    
   
    ostring=stringParameter("detmaskset")
    setO=dataset(ostring,Create)
! RDS
! RDS - change from PRIMARY array to MASK extension
!
!    marr=addArray(setO,"PRIMARY",Integer32,shape)
!
!  Create a new extension MASK
!    primary_block= block(setO, 0, MODIFY)

!    call copyBlock(setO, primary_block, "MASK") 
!
!    marr2=addArray(setO,"PRIMARY",Integer32,shape) ! Duplication
    marr=addArray(setO,"MASK",Integer32,shape)
!
! RDS - end of change

    x => int32Array2Data(marr)
    x = 1 

    emax = expo(0,0)
    do ix = 0,npix(1)-1
      do iy = 0,npix(2)-1
        emax = max(expo(ix,iy),emax)
      end do
    end do

    fracth = fracth*emax
    gradth = gradth*emax

    if (regflag) then
      do ireg=1,nreg
        ! pixel centres are at <nx>.0, <ny>.0:
        xreg(ireg)=xreg(ireg)-0.5
        yreg(ireg)=yreg(ireg)-0.5
        rreg2(ireg)=rreg(ireg)**2
        pa=pi/180.0*rotreg(ireg)
        sinp(ireg)=sin(pa)
        cosp(ireg)=cos(pa)
      enddo
    endif
     do ix = 0,npix(1)-1
      do iy = 0,npix(2)-1
        if (expo(ix,iy) < fracth) then
          x(ix,iy) = 0
        end if
        if (regflag) then
          do ireg=1,nreg
            if (regtype(ireg) == 1) then
              if (((float(ix)-xreg(ireg))**2+(float(iy)-yreg(ireg))**2 ) < rreg2(ireg)) then
                x(ix,iy) = 0
              endif
            elseif (regtype(ireg) == 2) then
              xr=(real(ix)-xreg(ireg))*cosp(ireg)+(real(iy)-yreg(ireg))*sinp(ireg)
              yr=(real(iy)-yreg(ireg))*cosp(ireg)-(real(ix)-xreg(ireg))*sinp(ireg)
              if (abs(xr) < w1reg(ireg) .and. abs(yr) < w2reg(ireg)) then
                x(ix,iy) = 0
              endif
            endif 
          enddo
        endif
      end do
    end do

     do ix = 1,npix(1)-2
       do iy = 1,npix(2)-2
        if (sqrt( ( expo(ix+1,iy)-expo(ix-1,iy) )**2 + &
                  ( expo(ix,iy+1)-expo(ix,iy-1) )**2 ) > gradth) then
           x(ix,iy) = 0
        end if 
      end do
    end do      

! RDS - copy header to MASK extension
    !maskBlock = block(setO, 1, MODIFY)
    call cp_head (set,marr,"EPIC DETECTION MASK",1)
!    call cp_head_primary (set,setO,"EPIC DETECTION MASK",1) ! Duplication

    call setAttribute(setO, "HDUCLASS", "OGIP    ", &
                            "Format conforms to OGIP/GSFC conventions")
    call setAttribute(setO, "HDUCLAS1", "IMAGE   ", &
                            "File contains an image")
    call setAttribute(setO, "HDUCLAS2", "MASK", &
                            "Detection mask")

! RDS - also add these attributes to the MASK block
    call setAttribute(marr, "HDUCLASS", "OGIP    ", &
                            "Format conforms to OGIP/GSFC conventions")
    call setAttribute(marr, "HDUCLAS1", "IMAGE   ", &
                            "File contains an image")
    call setAttribute(marr, "HDUCLAS2", "MASK", &
                            "Detection mask")
! RDS - end

! RDS - copy a few keywords from the MASK array to the PRIMARY header
    call setAttribute(setO,attribute(marr,"TELESCOP"))  
    call setAttribute(setO,attribute(marr,"INSTRUME"))  
    call setAttribute(setO,attribute(marr,"EXP_ID  "))  

! RDS - try and coipy mask array to PRIMARY extension - Duplication
!
!    x2 => int32Array2Data(marr2)
!
!     do ix = 0,npix(1)-1
!      do iy = 0,npix(2)-1
!         x2(ix,iy)=x(ix,iy)
!      end do
!     end do
! RDS - end of duplication

    call release(earr) 
    call release(set)

    call release(marr)
    call release(setO) 
         
  end subroutine emask
  
end module emask_mod 
   
